"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const jest_mock_server_1 = require("jest-mock-server");
const variables_manager_1 = require("../../managers/variables-manager");
const testing_1 = require("../../testing");
const get_1 = __importDefault(require("./get"));
const server = new jest_mock_server_1.MockServer();
let request;
beforeAll(() => server.start());
afterAll(() => server.stop());
beforeEach(() => {
    (0, testing_1.cleanTest)();
    request = testing_1.ElementTest.create(get_1.default);
    server.reset();
});
afterEach(async () => {
    await request.dispose();
});
test('send a "get" request then get json data', async () => {
    const bodyData = {
        name: 'thanh',
        sex: 'male'
    };
    server.get('/')
        .mockImplementationOnce((ctx) => {
        ctx.status = 200;
        ctx.message = 'OK MEN';
        ctx.body = bodyData;
        expect(ctx.query.time).toBeDefined();
        expect(ctx.query.rd).toBeDefined();
    });
    await request.init({
        url: server.getURL().toString() + '?time=' + Date.now(),
        query: {
            rd: Math.random().toString()
        },
        responseType: 'json',
        vars: {
            status: '${this.response.status}',
            statusText: '${this.response.statusText}',
            data: '${this.response.data}'
        }
    });
    await request.exec();
    expect(variables_manager_1.GlobalVars.status).toBe(200);
    expect(variables_manager_1.GlobalVars.statusText).toBe('OK MEN');
    expect(variables_manager_1.GlobalVars.data).toEqual(bodyData);
});
test('send a "get" request then get blob data', async () => {
    const bodyData = {
        name: 'thanh',
        sex: 'male'
    };
    server.get('/')
        .mockImplementationOnce((ctx) => {
        ctx.status = 200;
        ctx.message = 'OK MEN';
        ctx.body = bodyData;
        expect(ctx.query.time).toBeDefined();
        expect(ctx.query.rd).toBeDefined();
    });
    await request.init({
        url: server.getURL().toString() + '?time=' + Date.now(),
        query: {
            rd: Math.random().toString()
        },
        responseType: 'blob',
        vars: 'data'
    });
    await request.exec();
    expect(variables_manager_1.GlobalVars.data).toBeInstanceOf(Blob);
});
test('send a "get" request then get arraybuffer data', async () => {
    const bodyData = {
        name: 'thanh',
        sex: 'male'
    };
    server.get('/')
        .mockImplementationOnce((ctx) => {
        ctx.status = 200;
        ctx.message = 'OK MEN';
        ctx.body = bodyData;
        ctx.set('custom', 'thanh');
        expect(ctx.query.time).toBeDefined();
        expect(ctx.query.rd).toBeDefined();
    });
    await request.init({
        url: server.getURL().toString() + '?time=' + Date.now(),
        query: {
            rd: Math.random().toString()
        },
        responseType: 'buffer',
        vars: {
            status: '${this.response.status}',
            statusText: '${this.response.statusText}',
            data: '${this.response.data}',
            custom: '${this.response.headers?.custom}',
        }
    });
    await request.exec();
    expect(variables_manager_1.GlobalVars.status).toBe(200);
    expect(variables_manager_1.GlobalVars.statusText).toBe('OK MEN');
    expect(variables_manager_1.GlobalVars.custom).toBe('thanh');
    expect(variables_manager_1.GlobalVars.data).toBeInstanceOf(ArrayBuffer);
});
test('send a "get" request then ignore response data', async () => {
    const bodyData = {
        name: 'thanh',
        sex: 'male'
    };
    server.get('/')
        .mockImplementationOnce((ctx) => {
        ctx.status = 200;
        ctx.message = 'OK MEN';
        ctx.body = bodyData;
    });
    await request.init({
        url: server.getURL().toString(),
        responseType: 'none',
        vars: 'data'
    });
    await request.exec();
    expect(variables_manager_1.GlobalVars.data).toBeUndefined();
});
test('send a "get" request then get text data', async () => {
    const bodyData = {
        name: 'thanh',
        sex: 'male'
    };
    server.get('/')
        .mockImplementationOnce((ctx) => {
        ctx.status = 200;
        ctx.message = 'OK MEN';
        ctx.body = bodyData;
        expect(ctx.query.time).toBeDefined();
        expect(ctx.query.rd).toBeDefined();
    });
    await request.init({
        url: server.getURL().toString() + '?time=' + Date.now(),
        query: {
            rd: Math.random().toString()
        },
        responseType: 'text',
        vars: {
            status: '${this.response.status}',
            statusText: '${this.response.statusText}',
            data: '${this.response.data}'
        }
    });
    await request.exec();
    expect(variables_manager_1.GlobalVars.status).toBe(200);
    expect(variables_manager_1.GlobalVars.statusText).toBe('OK MEN');
    expect(typeof variables_manager_1.GlobalVars.data).toBe('string');
});
test('should download a file', async () => {
    const tmp = new testing_1.TmpFile();
    try {
        const bodyData = 'thanh';
        server.get('/')
            .mockImplementationOnce((ctx) => {
            ctx.status = 200;
            ctx.message = 'OK MEN';
            ctx.body = bodyData;
        });
        await request.init({
            url: server.getURL().toString(),
            saveTo: tmp.file
        });
        await request.exec();
        const fileContent = (0, fs_1.readFileSync)(tmp.file).toString();
        expect(fileContent).toBe(bodyData);
    }
    finally {
        tmp.remove();
    }
});
//# sourceMappingURL=get.spec.js.map