"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const jest_mock_server_1 = require("jest-mock-server");
const koa_1 = __importDefault(require("koa"));
const koa_body_1 = __importDefault(require("koa-body"));
const path_1 = require("path");
const variables_manager_1 = require("../../managers/variables-manager");
const testing_1 = require("../../testing");
const post_1 = __importDefault(require("./post"));
const server = new jest_mock_server_1.MockServer({
    buildApp() {
        const app = new koa_1.default();
        app.use((0, koa_body_1.default)({ multipart: true }));
        return app;
    }
});
let request;
beforeAll(() => server.start());
afterAll(() => server.stop());
beforeEach(() => {
    (0, testing_1.cleanTest)();
    request = testing_1.ElementTest.create(post_1.default);
    server.reset();
});
afterEach(async () => {
    await request.dispose();
});
test('post a json request ok', async () => {
    const bodyData = {
        name: 'thanh',
        sex: 'male'
    };
    server.post('/')
        .mockImplementationOnce((ctx) => {
        ctx.status = 200;
        ctx.message = 'OK MEN';
        ctx.body = ctx.request.body;
        expect(ctx.query.time).toBeDefined();
        expect(ctx.query.rd).toBeDefined();
    });
    await request.init({
        url: server.getURL().toString() + '?time=' + Date.now(),
        query: {
            rd: Math.random().toString()
        },
        responseType: 'json',
        type: 'json',
        body: bodyData,
        vars: {
            status: '${this.response.status}',
            statusText: '${this.response.statusText}',
            data: '${this.response.data}'
        }
    });
    await request.exec();
    expect(variables_manager_1.GlobalVars.status).toBe(200);
    expect(variables_manager_1.GlobalVars.statusText).toBe('OK MEN');
    expect(variables_manager_1.GlobalVars.data).toEqual(bodyData);
});
test('post a form request ok', async () => {
    const bodyData = {
        name: 'thanh',
        sex: 'male'
    };
    server.post('/')
        .mockImplementationOnce((ctx) => {
        ctx.status = 200;
        ctx.message = 'OK MEN';
        ctx.body = ctx.request.body;
    });
    await request.init({
        url: server.getURL().toString(),
        responseType: 'json',
        type: 'form',
        body: bodyData,
        vars: {
            status: '${this.response.status}',
            statusText: '${this.response.statusText}',
            data: '${this.response.data}'
        }
    });
    await request.exec();
    expect(variables_manager_1.GlobalVars.status).toBe(200);
    expect(variables_manager_1.GlobalVars.statusText).toBe('OK MEN');
    expect(variables_manager_1.GlobalVars.data).toEqual(bodyData);
});
test('post a text request ok', async () => {
    const bodyData = 'hello world';
    server.post('/')
        .mockImplementationOnce((ctx) => {
        expect(ctx.request.body).toBe(bodyData);
    });
    await request.init({
        url: server.getURL().toString(),
        responseType: 'none',
        type: 'text',
        body: bodyData
    });
    await request.exec();
});
test('post a raw request ok', async () => {
    const bodyData = 'hello world';
    server.post('/')
        .mockImplementationOnce((ctx) => {
        expect(ctx.request.body).toBe(bodyData);
    });
    await request.init({
        url: server.getURL().toString(),
        responseType: 'none',
        type: 'raw',
        body: bodyData,
        vars: {
            status: '${this.response.status}',
            statusText: '${this.response.statusText}',
            data: '${this.response.data}'
        }
    });
    await request.exec();
});
test('upload a file ok', async () => {
    const bodyData = {
        name: 'thanh',
        sex: 'male',
        avatar: {
            path: (0, path_1.resolve)(__dirname, './post.spec.ts'),
            name: 'my file.ts'
        }
    };
    server.post('/')
        .mockImplementationOnce((ctx) => {
        ctx.body = ctx.request.body;
        expect(ctx.request.body).toBeDefined();
        const { avatar, ...form } = bodyData;
        expect(ctx.request.body).toEqual(form);
        expect(ctx.request.files?.avatar).toBeDefined();
    });
    await request.init({
        url: server.getURL().toString(),
        responseType: 'json',
        type: 'multipart',
        body: bodyData,
    });
    await request.exec();
});
test('post a request without body ok', async () => {
    const bodyData = undefined;
    server.post('/')
        .mockImplementationOnce((ctx) => {
        ctx.body = 'ok';
    });
    await request.init({
        url: server.getURL().toString(),
        responseType: 'text',
        type: 'raw',
        body: bodyData,
    });
    await request.exec();
    expect(request.response.data).toBe('ok');
});
//# sourceMappingURL=post.spec.js.map