"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ansi_colors_1 = __importDefault(require("ansi-colors"));
const promises_1 = require("fs/promises");
const lodash_merge_1 = __importDefault(require("lodash.merge"));
const path_1 = require("path");
const aes_1 = require("../../libs/encrypt/aes");
const env_1 = require("../../libs/env");
const file_remote_1 = require("../../libs/file-remote");
const tags_manager_1 = require("../../managers/tags-manager");
const variables_manager_1 = require("../../managers/variables-manager");
const yaml_1 = require("yaml");
const element_utils_1 = require("../element.utils");
const group_1 = __importDefault(require("../group"));
const constants_1 = require("./constants");
class Scene {
    isRoot;
    title;
    description;
    ignoreVarsBeforeExec = ['content', '#group'];
    vars;
    path;
    encryptedPath;
    content;
    password;
    #group;
    logger;
    curDir;
    get count() {
        return this.#group.count;
    }
    on(event, cb) {
        return this.#group.on(event, cb);
    }
    constructor(isRoot = false) {
        this.isRoot = isRoot;
        this.#group = (0, element_utils_1.createElement)(group_1.default);
    }
    async init(props) {
        this.#group.logger = this.logger.clone('group');
        Object.assign(this, props);
        if (this.password)
            this.logger.context = 'scene/encrypted';
    }
    async getRemoteFileProps() {
        if (this.path) {
            const fileRemote = new file_remote_1.FileRemote(this.path);
            this.content = await fileRemote.getTextContent();
            if (!fileRemote.isRemote) {
                this.path = (0, path_1.resolve)(fileRemote.uri);
                const dirPath = (0, path_1.dirname)(this.path);
                if (this.isRoot)
                    variables_manager_1.GlobalUtils.rootDir = dirPath;
                variables_manager_1.GlobalUtils.curDir = this.curDir = dirPath;
            }
        }
        if (!this.content)
            throw new Error(`Scene file is not valid format`);
        const remoteFileProps = (0, yaml_1.parse)(await this.decryptContent(this.content, this.password));
        return remoteFileProps;
    }
    async decryptContent(content, password) {
        if (!password)
            return content;
        try {
            const encryptor = new aes_1.AES(`${constants_1.prefixPassword}${password}`);
            return encryptor.decrypt(content);
        }
        catch (err) {
            if (err?.code === 'ERR_OSSL_BAD_DECRYPT') {
                throw new Error(`Password to decrypt the file "${this.path}" is not valid`);
            }
            throw err;
        }
    }
    async generateEncryptedFile(content, password) {
        if (!password)
            return;
        this.encryptedPath = (0, path_1.join)(variables_manager_1.GlobalUtils.curDir, (0, path_1.basename)(this.path).split('.')[0]);
        this.logger.trace(`Encrypted to\t%s`, this.encryptedPath);
        const encryptor = new aes_1.AES(`${constants_1.prefixPassword}${password}`);
        const econtent = encryptor.encrypt(content);
        await (0, promises_1.writeFile)(this.encryptedPath, econtent);
    }
    async initGroup() {
        const remoteFileProps = await this.getRemoteFileProps();
        if (Array.isArray(remoteFileProps)) {
            this.path && this.logger.trace(`%s \t%s`, 'Scene', ansi_colors_1.default.italic(this.path));
            await this.#group.init(remoteFileProps);
        }
        else {
            let { title: _title, description: _description, log: _log, password: _password, vars: _vars, 'vars_file': _varsFile, tags: _tags, ...groupProps } = remoteFileProps;
            const { title, description = '', log, password, vars, varsFile, tags } = await (0, variables_manager_1.getVars)({ title: _title, description: _description, log: _log, password: _password, vars: _vars, varsFile: _varsFile, tags: _tags });
            if (log) {
                this.logger.setLevel(log);
                this.#group.logger.setLevel(log);
            }
            if (this.title === undefined) {
                this.title = title;
            }
            if (this.description === undefined) {
                this.description = description;
            }
            this.title && this.logger.log('%s', ansi_colors_1.default.cyan.bold(this.title));
            this.description && this.logger.log('%s', ansi_colors_1.default.gray.italic(this.description));
            this.path && this.logger.trace(`%s \t%s`, 'Scene', ansi_colors_1.default.italic(this.path));
            if (password && !this.password) {
                await this.generateEncryptedFile(this.content, password);
            }
            await this.loadVars(vars, varsFile);
            await tags_manager_1.tagsManager.init(tags, this.logger);
            await this.#group.init(groupProps);
        }
    }
    async loadVars(vars = {}, varsFile) {
        if (varsFile) {
            const file = new file_remote_1.FileRemote(varsFile);
            let content = await file.getTextContent();
            let newVars = {};
            try {
                newVars = JSON.parse(content);
            }
            catch {
                newVars = (0, yaml_1.parse)(content);
            }
            (0, lodash_merge_1.default)(vars, newVars);
        }
        (0, lodash_merge_1.default)(variables_manager_1.GlobalVars, vars);
        if (this.isRoot)
            await this.loadEnv();
        if (this.vars) {
            const overridedVars = await (0, variables_manager_1.getVars)(this.vars);
            (0, lodash_merge_1.default)(variables_manager_1.GlobalVars, overridedVars);
        }
        this.logger.trace(`Vars    \t%j`, variables_manager_1.GlobalVars);
    }
    async loadEnv(...envFiles) {
        const env = new env_1.Env(this.logger);
        await env.loadEnvToBase(variables_manager_1.GlobalVars, ...envFiles.filter(e => e), process.env);
    }
    async exec() {
        await this.initGroup();
        if (this.isRoot)
            this.logger.log(ansi_colors_1.default.yellow('⚡️'));
        return this.#group.exec();
    }
    async dispose() {
        return this.#group.dispose();
    }
}
exports.default = Scene;
//# sourceMappingURL=scene.js.map