"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const variables_manager_1 = require("../../managers/variables-manager");
const testing_1 = require("../../testing");
const yaml_1 = require("yaml");
const scene_1 = __importDefault(require("./scene"));
let scene;
let encryptedFile;
const password = 'example';
beforeEach(async () => {
    (0, testing_1.cleanTest)();
    scene = testing_1.ElementTest.create(scene_1.default);
});
afterEach(async () => {
    await scene.dispose();
});
afterAll(() => {
    encryptedFile && (0, fs_1.unlinkSync)(encryptedFile);
});
test('Should load vars from yaml file', async () => {
    scene.dispose();
    const tmpFile = new testing_1.TmpFile('.yaml');
    try {
        await tmpFile.create((0, yaml_1.stringify)({
            name: 'name 1',
            age: 1,
            male: true,
            more: {
                address: '1',
                num: 2,
                married: false
            }
        }));
        scene = testing_1.ElementTest.create(scene_1.default, true);
        await scene.init({
            content: `
vars_file: ${tmpFile.file}
runs:
  - Test env
`
        });
        await scene.exec();
        expect(variables_manager_1.GlobalVars.name).toBe('name 1');
        expect(variables_manager_1.GlobalVars.age).toBe(1);
        expect(variables_manager_1.GlobalVars.male).toBe(true);
        expect(variables_manager_1.GlobalVars.more).toEqual({
            address: '1',
            num: 2,
            married: false
        });
    }
    finally {
        tmpFile.remove();
    }
});
test('Should load vars from json file', async () => {
    scene.dispose();
    const tmpFile = new testing_1.TmpFile('.json');
    try {
        await tmpFile.create(JSON.stringify({
            name: 'name 1',
            age: 1,
            male: true,
            more: {
                address: '1',
                num: 2,
                married: false
            }
        }));
        scene = testing_1.ElementTest.create(scene_1.default, true);
        await scene.init({
            content: `
vars_file: ${tmpFile.file}
runs:
  - Test env
`
        });
        await scene.exec();
        expect(variables_manager_1.GlobalVars.name).toBe('name 1');
        expect(variables_manager_1.GlobalVars.age).toBe(1);
        expect(variables_manager_1.GlobalVars.male).toBe(true);
        expect(variables_manager_1.GlobalVars.more).toEqual({
            address: '1',
            num: 2,
            married: false
        });
    }
    finally {
        tmpFile.remove();
    }
});
test('Should override env when load scene', async () => {
    scene.dispose();
    process.env.NAME = 'name 1';
    process.env.age = '1';
    process.env.male = '1';
    process.env.more_address = '1';
    process.env.MORE_NUM = '2';
    process.env.MORE_MARRied = 'no';
    scene = testing_1.ElementTest.create(scene_1.default, true);
    await scene.init({
        content: `
vars:
  name: name 0
  age: 0
  male: false
  more:
    address: addr 1
    num: 1
    married: true
runs:
  - Test env
`
    });
    await scene.exec();
    expect(variables_manager_1.GlobalVars.name).toBe('name 1');
    expect(variables_manager_1.GlobalVars.age).toBe(1);
    expect(variables_manager_1.GlobalVars.male).toBe(true);
    expect(variables_manager_1.GlobalVars.more).toEqual({
        address: '1',
        num: 2,
        married: false
    });
});
test('Should load successfully from text content with only list items', async () => {
    expect(scene.count).toBe(0);
    await scene.init({ content: `- echo: Hello` });
    await scene.exec();
    expect(scene.count).toBe(1);
});
test('Should load successfully from text content with full props', async () => {
    expect(scene.count).toBe(0);
    await scene.init({
        content: `
title: test
runs: 
  - echo: Hello
`
    });
    await scene.exec();
    expect(scene.count).toBe(1);
});
test('Should load successfully from file', async () => {
    const tmp = new testing_1.TmpFile();
    try {
        await tmp.create(`
title: Test here
runs:
  - Hello
  - World
`);
        expect(scene.count).toBe(0);
        await scene.init({ path: tmp.file });
        await scene.exec();
        expect(scene.count).toBe(2);
    }
    finally {
        tmp.remove();
    }
});
test('Should generate a generated scene file when set password', async () => {
    const tmp = new testing_1.TmpFile('.yaml');
    try {
        await tmp.create(`
title: Test here
password: ${password}
runs:
  - Hello
  - World
`);
        await scene.init({ path: tmp.file });
        await scene.exec();
        expect((0, fs_1.existsSync)(scene.encryptedPath)).toBe(true);
        encryptedFile = scene.encryptedPath;
    }
    finally {
        tmp.remove();
    }
});
test('Load a encrypted scene file', async () => {
    expect(scene.count).toBe(0);
    await scene.init({ path: encryptedFile, password });
    await scene.exec();
    expect(scene.count).toBe(2);
});
test('Should run echo element', async () => {
    let echo;
    scene.on('item/created', item => {
        echo = item;
        echo.exec = jest.fn();
        echo.dispose = jest.fn();
    });
    await scene.init({
        content: `
- echo:
    content: 'ok'
`
    });
    await scene.exec();
    expect(echo.exec).toBeCalledTimes(1);
    expect(echo.dispose).toBeCalledTimes(1);
    await scene.dispose();
    expect(echo.dispose).toBeCalledTimes(1);
});
//# sourceMappingURL=scene.spec.js.map