"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = exports.LoggerLevel = void 0;
const ansi_colors_1 = __importDefault(require("ansi-colors"));
var LoggerLevel;
(function (LoggerLevel) {
    LoggerLevel["ALL"] = "trace";
    LoggerLevel["TRACE"] = "trace";
    LoggerLevel["DEBUG"] = "debug";
    LoggerLevel["INFO"] = "info";
    LoggerLevel["WARN"] = "warn";
    LoggerLevel["ERROR"] = "error";
    LoggerLevel["SILENT"] = "silent";
})(LoggerLevel = exports.LoggerLevel || (exports.LoggerLevel = {}));
const mapping = {
    all: 0,
    trace: 2,
    debug: 4,
    info: 6,
    warn: 8,
    error: 10,
    silent: 12,
};
class Logger {
    context;
    indent;
    level;
    indentString = '';
    constructor(level, context = '', indent = 0) {
        this.context = context;
        this.indent = indent;
        this.setLevel(level);
        indent && this.addIndent(0);
    }
    is(level) {
        const lv = typeof level === 'number' ? level : mapping[level];
        return this.level <= lv;
    }
    clone(context) {
        return new Logger(this.level, context, this.indent);
    }
    addIndent(indent) {
        this.indent += indent;
        this.indentString = this.getIndentString(this.indent);
    }
    getIndentString(indent) {
        return new Array(indent).fill('  ').join('');
    }
    formatWithoutIndent(msg, level) {
        switch (level) {
            case LoggerLevel.INFO:
                return `${ansi_colors_1.default.green(`✔`)} ${msg}`;
            case LoggerLevel.WARN:
                return `${ansi_colors_1.default.yellow(`⚠️`)} ${msg}`;
            case LoggerLevel.ERROR:
                return `${ansi_colors_1.default.yellow(`✘`)} ${msg}`;
        }
        return `${msg}`;
    }
    format(msg, level) {
        return `${this.indentString}${this.formatWithoutIndent(msg, level)}`;
    }
    setLevel(level) {
        this.level = typeof level === 'number' ? level : (mapping[level] || mapping.all);
    }
    log(msg, ...prms) {
        this.is(mapping.error) && console.log(`${this.indentString}${msg}`, ...prms);
    }
    info(msg, ...prms) {
        this.is(mapping.info) && console.log(`${this.indentString}%s ${msg} \t %s`, ansi_colors_1.default.green(`✔`), ...prms, !this.context ? '' : ansi_colors_1.default.italic.gray.dim(`[${this.context}]`));
    }
    debugBlock(isStart, msg = '', ...prms) {
        if (!this.is(mapping.debug))
            return;
        if (!isStart)
            console.groupEnd();
        console.log(`${this.indentString}%s ${msg}`, ansi_colors_1.default.gray(isStart ? '┌' : '└'), ...prms);
        if (isStart)
            console.group();
    }
    debug(msg, ...prms) {
        this.is(mapping.debug) && console.log(`${this.indentString}${ansi_colors_1.default.gray(msg)} \t %s`, ...prms, !this.context ? '' : ansi_colors_1.default.italic.gray.dim(`[${this.context}]`));
    }
    warn(msg, ...prms) {
        this.is(mapping.warn) && console.log(`${this.indentString}%s ${ansi_colors_1.default.yellow(msg)} \t %s`, ansi_colors_1.default.yellow('⚠️'), ...prms, !this.context ? '' : ansi_colors_1.default.italic.gray.dim(`[${this.context}]`));
    }
    trace(msg, ...prms) {
        this.is(mapping.trace) && console.log(`${this.indentString}%s ${ansi_colors_1.default.magenta(msg)} \t %s`, ansi_colors_1.default.magenta('⊹'), ...prms, !this.context ? '' : ansi_colors_1.default.italic.gray.dim(`[${this.context}]`));
    }
    error(msg, ...prms) {
        this.is(mapping.error) && console.log(`${this.indentString}%s ${msg} \t %s`, ansi_colors_1.default.red(`✘`), ...prms, !this.context ? '' : ansi_colors_1.default.italic.gray.dim(`[${this.context}]`));
    }
    fatal(msg, ...prms) {
        this.is(mapping.error) && console.log(`${this.indentString}%s ${msg} \t %s`, ansi_colors_1.default.red(`❌`), ...prms, !this.context ? '' : ansi_colors_1.default.italic.gray.dim(`[${this.context}]`));
    }
}
exports.Logger = Logger;
//# sourceMappingURL=logger.js.map